//=============================================================================
// SAN_AnalogStick.js
//=============================================================================
// Copyright (c) 2017 Sanshiro
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
//=============================================================================

/*:
 * @plugindesc analog stick 1.0.0
 * Get the input of the analog stick.
 * @author サンシロ https://twitter.com/rev2nym
 * @version 1.0.0 2017/03/01 creation
 * 
 * @help
 * ■Overview
 * Enables the input value of the gamepad analog stick to be used.
 * The game pad with the highest tilt among the connected game pads.
 * Adopt analog stick values.
 * 
 * ■Getting input values
 * The following script will retrieve the input values.
 * 
 * ・Direction value of the left stick
 *   Input.leftStick.dir
 * 
 * ・Tilt value of the left stick
 *   Input.leftStick.tilt
 * 
 * ・Direction value of the right stick
 *   Input.rightStick.dir
 * 
 * ・Tilt value of the right stick
 *   Input.rightStick.tilt
 * 
 * The directional values are in the radian system, with the right direction being 0.0 and the clockwise direction being positive.
 * The slope value of the stick is normalized between 0.0 and 1.0.
 * Values less than or equal to 0.2 are rounded down to 0.0 for fun.
 * 
 * For example, with the following input values, the state of the stick is
 *   {dir:   0.0, tilt: 1.0}  // Full to the right.
 *   {dir:  3.14, tilt: 0.5}  // Half to the left.
 *   {dir:  1.57, tilt: 1.0}  // Full downward.
 *   {dir: -1.57, tilt: 0.5}  // Halfway up.
 * 
 * ■Terms of Use
 * It can be used, modified, and redistributed commercially under the MIT license.
 * However, please do not delete or alter the comments at the beginning.
 * 
 * The author is not responsible for any damage caused by the use of this site.
 * Don't expect any support >.<
 */

var Imported = Imported || {};
Imported.SAN_AnalogStick = true;

var Sanshiro = Sanshiro || {};
Sanshiro.AnalogStick = Sanshiro.AnalogStick || {};
Sanshiro.AnalogStick.version = '1.0.0';

(function() {
'use strict';

//-----------------------------------------------------------------------------
// Input
//
// input class

// Gamepad analog stick
Input.leftStick = {dir: 0.0, tilt: 0.0}; // left stick
Input.rightStick = {dir: 0.0, tilt: 0.0}; // right stick

Input._stickThreshold = 0.2; // Input detection threshold (play)

// Frame update
var _Input_update = Input.update;
Input.update = function() {
    _Input_update.call(this);
    this._updateSticks();
};

// Update gamepad analog sticks.
Input._updateSticks = function() {
    var axes = [0.0, 0.0, 0.0, 0.0];
    if (!!navigator.getGamepads && !!navigator.getGamepads()) {
        // Adopt the analog stick value with the highest tilt among the connected gamepads.
        var gamepads = navigator.getGamepads();
        for (var i = 0; i < gamepads.length; i++) {
            var gamepad = gamepads[i];
            if (!gamepad || !gamepad.connected) {
                continue;
            }
            if (Math.pow(axes[0], 2) + Math.pow(axes[1], 2) + 
                Math.pow(axes[2], 2) + Math.pow(axes[3], 2) <
                Math.pow(gamepad.axes[0], 2) + Math.pow(gamepad.axes[1], 2) +
                Math.pow(gamepad.axes[2], 2) + Math.pow(gamepad.axes[3], 2))
            {
                axes = gamepad.axes;
            }
        }
    }
    // left stick
    this.leftStick.dir = this._stickDir(axes[0], axes[1]);
	
	// clamp right
	if ( (this.leftStick.dir <= 0.37) && (this.leftStick.dir >= -0.37) ) {
		this.leftStick.dir = 0;
	}
	
	// clamp down
	if ( (this.leftStick.dir <= 1.94) && (this.leftStick.dir >= 1.20) ) {
		this.leftStick.dir = 1.57;
	}
	
	// clamp up
	if ( (this.leftStick.dir >= -1.94) && (this.leftStick.dir <= -1.20) ) {
		this.leftStick.dir = -1.57;
	}
	
	// clamp left
	if ( (this.leftStick.dir >= 2.77) || (this.leftStick.dir <= -2.77) ) {
		this.leftStick.dir = 3.14;
	}
	
    this.leftStick.tilt = this._stickTilt(axes[0], axes[1]);
    // right stick
    this.rightStick.dir = this._stickDir(axes[2], axes[3]);
    this.rightStick.tilt = this._stickTilt(axes[2], axes[3]);
};

// Analog stick tilt
Input._stickTilt = function(x, y) {
    var tilt = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
    var tTh  = this._stickThreshold; // Input detection threshold
    return (
        tilt < tTh ? 0.0 :
        tilt > 1.0 ? 1.0 : tilt
    );
};

// Analog stick direction
Input._stickDir = function(x, y) {
    return Math.atan2(y, x);
};

})();
